# @Time    : 2023/8/4 10:34
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : QLType.py
from ultipa.proto import ultipa_pb2
class QLType:
    '''
    	    Data class for QL type
    	'''
    UQL = ultipa_pb2.UQL
    GQL = ultipa_pb2.GQL